package com.gree.mall.contest.controller.pc.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.annotation.ApiNotAuth;
import com.gree.mall.contest.bean.SVerification;
import com.gree.mall.contest.bean.admin.AdminModuleTree;
import com.gree.mall.contest.bean.admin.AdminUserBean;
import com.gree.mall.contest.bean.admin.AdminWebsitGrantBean;
import com.gree.mall.contest.bean.admin.reqDto.AdminUserAddReqBean;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.admin.AdminUserLogic;
import com.gree.mall.contest.plus.entity.AdminUser;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

@Slf4j
@RestController
@Tag(name = "系统用户管理", description = "后台用户相关API")
@RequestMapping(value = "/pc/admin/user", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class AdminUserController {

    private final AdminUserLogic adminUserLogic;
    private final RedisLockRegistry redisLockRegistry;

    @ApiNotAuth
    @PostMapping("/login")
    @Operation(summary = "登录")
    public ResponseHelper<AdminUserBean> login(
            @Parameter(description = "帐号",required = true) @RequestParam(required = true) String userName,
            @Parameter(description = "密码",required = false) @RequestParam(required = false) String password,
            @Parameter(description = "验证码code",required = false) @RequestParam(required = false) String code,
            @Parameter(description = "验证码值",required = false) @RequestParam(required = false) String codeValue

    ) throws RemoteServiceException, InterruptedException {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_LOGIN + userName);
        if(!obtain.tryLock(10, TimeUnit.SECONDS)){
            throw new RemoteServiceException("系统繁忙请稍后再试");
        }
        try {
            AdminUserBean adminUserBean = adminUserLogic.login(userName, password, code, codeValue);
            return ResponseHelper.success(adminUserBean);
        }finally {
            obtain.unlock();
        }
    }

    @PostMapping("/password/update")
    @Operation(summary = "修改密码")
    public ResponseHelper<AdminUserBean> login(
            @Parameter(description = "帐号",required = true) @RequestParam(required = true) String userName,
            @Parameter(description = "密码",required = true) @RequestParam(required = true) String password,
            @Parameter(description = "新密码",required = true) @RequestParam(required = true) String newPassword
    ) throws RemoteServiceException {
        adminUserLogic.updatePassword(userName, password,newPassword);
        return ResponseHelper.success();
    }

    @PostMapping("/status/update")
    @Operation(summary = "修改帐号状态(冻结/正常)")
    public ResponseHelper updateStatus(
            @Parameter(description = "帐号名称",required = true) @RequestParam(required = true) String adminUserId,
            @Parameter(description = "true:正常  false:冻结",required = true) @RequestParam(required = true) Boolean status
    ) throws RemoteServiceException {
        adminUserLogic.updateStatus(adminUserId,status);
        return ResponseHelper.success();
    }


    @PostMapping("/add")
    @Operation(summary = "新增帐号")
    public ResponseHelper add(
            @Parameter(description = "帐号",required = true) @RequestBody AdminUserAddReqBean adminUserBean
    ) throws RemoteServiceException {
        adminUserLogic.add(adminUserBean);
        return ResponseHelper.success();
    }


    @PostMapping("/user/update")
    @Operation(summary = "修改帐号")
    public ResponseHelper updateUser(
            HttpServletRequest request,
            @Parameter(description = "帐号", required = true) @RequestBody AdminUserAddReqBean adminUserBean
    ) throws RemoteServiceException {
        adminUserLogic.update(request, adminUserBean);
        return ResponseHelper.success();
    }

    @PostMapping("/password/reset")
    @Operation(summary = "重置密码")
    public ResponseHelper resetPwd(
            @Parameter(description = "用户id",required = true) @RequestParam(required = true) String adminUserId,
            @Parameter(description = "密码",required = true) @RequestParam(required = true) String password,
            HttpServletRequest request
    ) throws RemoteServiceException {
        adminUserLogic.resetPassword(adminUserId,password,request);
        return ResponseHelper.success();
    }


    @GetMapping("/list")
    @Operation(summary = "帐号列表")
    public ResponseHelper<IPage<AdminUser>> list(
            @Parameter(description = "部门id",required = false) @RequestParam(required = false) String adminWebsitId,
            @Parameter(description = "角色id",required = false) @RequestParam(required = false) String roleId,
            @Parameter(description = "状态 true:正常 false:冻结",required = false) @RequestParam(required = false) Boolean status,
            @Parameter(description = "用户名",required = false) @RequestParam(required = false) String userName,
            @Parameter(description = "页号",required = true) @RequestParam(required = true) Integer pageNum,
            @Parameter(description = "页大小",required = true) @RequestParam(required = true) Integer pageSize,
            HttpServletRequest request
    ) throws RemoteServiceException {
        IPage<AdminUser> page = adminUserLogic.list(adminWebsitId,roleId,status,userName,pageNum, pageSize);
        return ResponseHelper.success(page);
    }

    @GetMapping("/module/list")
    @Operation(summary = "功能菜单权限列表")
    public ResponseHelper<List<AdminModuleTree>> moduleList(
            @Parameter(description = "用户id",required = true) @RequestParam(required = true) String adminUserId
    ) throws RemoteServiceException {
        List<AdminModuleTree> adminModuleTrees = adminUserLogic.queryAdminModule(adminUserId);
        return ResponseHelper.success(adminModuleTrees);
    }

    @GetMapping("/module/id/checked")
    @Operation(summary = "查询选中的功能模块ids")
    public ResponseHelper<List<String>> queryModuleIdChecked(
            @Parameter(description = "角色id",required = true) @RequestParam(required = true) String adminRoleId
    ) throws RemoteServiceException {
        List<String> moduleIds = adminUserLogic.queryModuleIdChecked(adminRoleId);
        return ResponseHelper.success(moduleIds);
    }

    @GetMapping("/module/all")
    @Operation(summary = "全部功能菜单权限列表")
    public ResponseHelper<List<AdminModuleTree>> allModuleList(
            @Parameter(description = "角色",required = true) @RequestParam(required = true) String adminRoleId,
            HttpServletRequest request
    ) throws RemoteServiceException {
        List<AdminModuleTree> adminModuleTrees = adminUserLogic.queryAllAdminModuleTree(adminRoleId,request);
        return ResponseHelper.success(adminModuleTrees);
    }

    @PostMapping("/module/grant")
    @Operation(summary = "授权动能菜单权限")
    public ResponseHelper grant(
            @RequestBody AdminWebsitGrantBean adminWebsitGrantBean
            ) throws RemoteServiceException {
        adminUserLogic.grantModules(adminWebsitGrantBean);
        return ResponseHelper.success();
    }

    @GetMapping("/detail")
    @Operation(summary = "详情")
    public ResponseHelper<AdminUserBean> detail(
            @Parameter(description = "帐号id",required = true) @RequestParam(required = true) String adminUserId
    ) throws RemoteServiceException {
        AdminUserBean detail = adminUserLogic.detail(adminUserId);
        return ResponseHelper.success(detail);
    }

    @PostMapping("/logout")
    @Operation(summary = "退出登录")
    public ResponseHelper logout(HttpServletRequest request){
        adminUserLogic.logout(request);
        return ResponseHelper.success();
    }

    @ApiNotAuth
    @GetMapping("/imageVerification")
    @Operation(summary = "获取图片验证码")
    public ResponseHelper<SVerification> defaultKaptcha()
            throws Exception {
        SVerification sVerification = adminUserLogic.defaultKaptcha();
        return ResponseHelper.success(sVerification);
    }

    @ApiNotAuth
    @PostMapping("/reset/password/getCode")
    @Operation(summary = "找回密码-获取验证码")
    public ResponseHelper<AdminUserBean> resetPasswordGetCode (
            @Parameter(description = "账号",required = true) @RequestParam(required = true) String userName,
            @Parameter(description = "手机号",required = true) @RequestParam String mobile,
            @Parameter(description = "短信验证码",required = true) @RequestParam String code
    ) throws RemoteServiceException {
        AdminUserBean bean = adminUserLogic.resetPasswordGetCode(userName,mobile, code);
        return ResponseHelper.success(bean);
    }

    @ApiNotAuth
    @PostMapping("/reset/password/update")
    @Operation(summary = "找回密码-更新密码")
    public ResponseHelper resetPasswordUpdate (
            @Parameter(description = "系统用户id",required = true) @RequestParam String adminUserId,
            @Parameter(description = "token",required = true) @RequestParam String token,
            @Parameter(description = "新密码",required = true) @RequestParam String newPassword,
            @Parameter(description = "确认密码",required = true) @RequestParam String confirmPassword
    ) throws RemoteServiceException {
        adminUserLogic.resetPasswordUpdate(adminUserId, token, newPassword, confirmPassword);
        return ResponseHelper.success();
    }

    @PostMapping("/service/update/mp/mobile")
    @Operation(summary = "服务商-更新公众号接收手机号")
    public ResponseHelper serviceUpdateMpMobile (
            @Parameter(description = "手机号",required = true) @RequestParam String mobile
    ) throws RemoteServiceException {
        adminUserLogic.serviceUpdateMpMobile(mobile);
        return ResponseHelper.success();
    }

    @Operation(summary = "续期token")
    @PostMapping("/renewal")
    public ResponseHelper<AdminUserBean> renewal(
            @RequestParam String token
    ) throws RemoteServiceException {
        AdminUserBean adminUserBean = adminUserLogic.renewal(token);
        return ResponseHelper.success(adminUserBean);
    }
}
